@@ -1,4 +1,3 @@
-# coding=utf-8
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
@@ -42,7 +41,7 @@ def test_iomanager_02(report_size, iters):
             size = len(iom.tests)
             assert precommit_size <= size
             assert size <= report_size
-        assert size == report_size
+        assert len(iom.tests) == report_size
         iom.purge()
         assert iom._test is None
         assert not iom.tests
