@@ -1,4 +1,3 @@
-# coding=utf-8
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
@@ -50,21 +49,18 @@ def _load_strategies():
         try:
             strategy_cls = entry_point.load()
             strategy_cls.sanity_check_cls_attrs()
-            assert (
-                strategy_cls.name == entry_point.name
-            ), "entry_point name mismatch, check setup.py and %s.name" % (
-                strategy_cls.__name__,
+            assert strategy_cls.name == entry_point.name, (
+                f"entry_point name mismatch, check setup.py and "
+                f"{strategy_cls.__name__.name}"
             )
         except Exception as exc:  # pylint: disable=broad-except
             LOG.debug("error loading strategy type %s: %s", entry_point.name, exc)
             continue
         strategies[entry_point.name] = strategy_cls
     for strategy in DEFAULT_STRATEGIES:
-        assert (
-            strategy in strategies
-        ), "Unknown entry in DEFAULT_STRATEGIES: %s (STRATEGIES: [%s])" % (
-            strategy,
-            ",".join(strategies),
+        assert strategy in strategies, (
+            f"Unknown entry in DEFAULT_STRATEGIES: {strategy} "
+            f"(STRATEGIES: [{','.join(strategies)}])"
         )
     return MappingProxyType(strategies)
 
@@ -165,7 +161,7 @@ class Strategy(ABC):
             self._testcase_root.mkdir()
         for idx, testcase in enumerate(testcases):
             LOG.debug("Extracting testcase %d/%d", idx + 1, len(testcases))
-            testpath = self._testcase_root / ("%03d" % (idx,))
+            testpath = self._testcase_root / f"{idx:0>3d}"
             testcase.dump(str(testpath), include_details=True)
 
     @classmethod
