@@ -1,4 +1,3 @@
-# coding=utf-8
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
@@ -70,7 +69,7 @@ class MinimizeTestcaseList(Strategy):
         idx = 0
         testcases = None
         try:
-            testcases = TestCase.load(str(self._testcase_root), True)
+            testcases = TestCase.load(self._testcase_root, True)
             n_testcases = len(testcases)
             # indicates that self._testcase_root contains changes that haven't been
             # yielded (if iteration ends, changes would be lost)
@@ -87,7 +86,7 @@ class MinimizeTestcaseList(Strategy):
                     break
                 # try removing the testcase at idx
                 if testcases is None:
-                    testcases = TestCase.load(str(self._testcase_root), True)
+                    testcases = TestCase.load(self._testcase_root, True)
                     assert n_testcases == len(testcases)
                 testcases.pop(idx).cleanup()
                 yield testcases
@@ -99,7 +98,7 @@ class MinimizeTestcaseList(Strategy):
                     LOG.info(
                         "Removing testcase %d/%d was successful!", idx + 1, n_testcases
                     )
-                    testcases = TestCase.load(str(self._testcase_root), True)
+                    testcases = TestCase.load(self._testcase_root, True)
                     try:
                         # remove the actual testcase we were reducing
                         testcases.pop(idx).cleanup()
@@ -112,7 +111,7 @@ class MinimizeTestcaseList(Strategy):
                     finally:
                         for testcase in testcases:
                             testcase.cleanup()
-                    testcases = TestCase.load(str(self._testcase_root), True)
+                    testcases = TestCase.load(self._testcase_root, True)
                     n_testcases = len(testcases)
                 else:
                     LOG.info("No result without testcase %d/%d", idx + 1, n_testcases)
@@ -124,7 +123,7 @@ class MinimizeTestcaseList(Strategy):
                 # purging unserved files enabled us to exit early from the loop.
                 # need to yield once more to set this trimmed version to the current
                 # best in ReduceManager
-                testcases = TestCase.load(str(self._testcase_root), True)
+                testcases = TestCase.load(self._testcase_root, True)
                 LOG.info("[%s] final iteration triggered by purge_optional", self.name)
                 yield testcases
                 testcases = None  # caller owns testcases now
