@@ -1,4 +1,3 @@
-# coding=utf-8
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
@@ -7,8 +6,8 @@ from time import time
 
 from .common.iomanager import IOManager
 from .common.runner import Runner
-from .common.status import Status
-from .target import Result, TargetLaunchError
+from .common.status import STATUS_DB_FUZZ, Status
+from .target import Result
 
 __all__ = ("SessionError", "LogOutputLimiter", "Session")
 __author__ = "Tyson Smith"
@@ -99,6 +98,7 @@ class Session:
         self.reporter = reporter
         self.server = server
         self.status = Status.start(
+            STATUS_DB_FUZZ,
             enable_profiling=enable_profiling,
             report_limit=report_limit,
         )
@@ -111,6 +111,9 @@ class Session:
         self.close()
 
     def close(self):
+        if self.status is not None:
+            # perform final status report
+            self.status.report(force=True)
         if self.iomanager is not None:
             self.iomanager.cleanup()
 
@@ -129,7 +132,7 @@ class Session:
     def generate_testcase(self, time_limit):
         LOG.debug("calling iomanager.create_testcase()")
         test = self.iomanager.create_testcase(self.adapter.name, time_limit)
-        LOG.debug("calling self.adapter.generate()")
+        LOG.debug("calling adapter.generate()")
         with self.status.measure("generate"):
             self.adapter.generate(test, self.iomanager.server_map)
         self.status.test_name = test.input_fname
@@ -141,6 +144,7 @@ class Session:
         time_limit,
         input_path=None,
         iteration_limit=0,
+        no_harness=False,
         result_limit=0,
         runtime_limit=0,
         display_mode=DISPLAY_NORMAL,
@@ -153,13 +157,15 @@ class Session:
 
         LOG.debug("calling adapter.setup()")
         self.adapter.setup(input_path, self.iomanager.server_map)
-        LOG.debug("configuring harness")
-        harness = self.adapter.get_harness()
+        LOG.debug("configuring harness (%r)", not no_harness)
+        harness = None if no_harness else self.adapter.get_harness()
+        LOG.debug("configuring redirects (w/harness: %s)", harness is not None)
         if harness is None:
             self.iomanager.server_map.set_redirect(
                 "grz_start", "grz_current_test", required=False
             )
         else:
+            assert harness
             self.iomanager.server_map.set_dynamic_response(
                 "grz_harness", lambda _: harness, mime_type="text/html", required=False
             )
@@ -183,34 +189,14 @@ class Session:
                 # (re-)launch target
                 self.iomanager.purge()
                 self.adapter.pre_launch()
-                if harness is None:
-                    # harness is not in use, open the test case
-                    location = runner.location("/grz_start", self.server.port)
-                else:
-                    # harness is in use, open it and it will open the test case.
-                    location = runner.location(
-                        "/grz_start",
-                        self.server.port,
-                        close_after=relaunch,
-                        time_limit=time_limit,
-                    )
-                try:
-                    with self.status.measure("launch"):
-                        runner.launch(
-                            location, max_retries=launch_attempts, retry_delay=0
-                        )
-                except TargetLaunchError as exc:
-                    short_sig = exc.report.crash_info.createShortSignature()
-                    LOG.info(
-                        "Result: %s (%s:%s)",
-                        short_sig,
-                        exc.report.major[:8],
-                        exc.report.minor[:8],
-                    )
-                    self.status.results.count(exc.report.crash_hash, short_sig)
-                    self.reporter.submit([], exc.report)
-                    exc.report.cleanup()
-                    raise TargetLaunchError(str(exc), None) from None
+                location = runner.location(
+                    "/grz_start",
+                    self.server.port,
+                    close_after=relaunch if harness else None,
+                    time_limit=time_limit if harness else None,
+                )
+                with self.status.measure("launch"):
+                    runner.launch(location, max_retries=launch_attempts, retry_delay=0)
 
             # create and populate a test case
             current_test = self.generate_testcase(time_limit)
@@ -230,10 +216,10 @@ class Session:
             # adapter callbacks
             if result.timeout:
                 current_test.hang = True
-                LOG.debug("calling self.adapter.on_timeout()")
+                LOG.debug("calling adapter.on_timeout()")
                 self.adapter.on_timeout(current_test, result.served)
             else:
-                LOG.debug("calling self.adapter.on_served()")
+                LOG.debug("calling adapter.on_served()")
                 self.adapter.on_served(current_test, result.served)
             if not result.attempted:
                 LOG.warning("Test case was not served")
@@ -284,7 +270,14 @@ class Session:
                 report.cleanup()
             elif result.status == Result.IGNORED:
                 self.status.ignored += 1
-                LOG.info("Ignored - %d", self.status.ignored)
+                if result.timeout:
+                    LOG.info(
+                        "Ignored - %d; timeout, idle: %r",
+                        self.status.ignored,
+                        result.idle,
+                    )
+                else:
+                    LOG.info("Ignored - %d", self.status.ignored)
 
             # ignore startup failure if it did not happen early on
             # to avoid aborting the fuzzing session unnecessarily
@@ -314,6 +307,3 @@ class Session:
                 LOG.warning(
                     "Large browser logs: %dMBs", (self.status.log_size / 0x100000)
                 )
-
-        # perform final status report
-        self.status.report(force=True)
