@@ -141,9 +141,9 @@ def test_puppet_target_04(mocker, tmp_path, healthy, usage, os_name, killed):
     assert fake_kill.call_count == fake_ffp.return_value.wait.call_count == killed
 
 
-@mark.skipif(system() == "Windows", reason="Unsupported on Windows")
+@mark.skipif(system() != "Linux", reason="Linux only")
 def test_puppet_target_05(mocker, tmp_path):
-    """test PuppetTarget.dump_coverage()"""
+    """test PuppetTarget.dump_coverage() - full test"""
     mocker.patch("grizzly.target.puppet_target.wait_procs", autospec=True)
     fake_ffp = mocker.patch("grizzly.target.puppet_target.FFPuppet", autospec=True)
     child_proc = mocker.Mock(pid=101)
@@ -335,7 +335,9 @@ def test_puppet_target_08(mocker, tmp_path):
         (False, False, True),
     ],  # pylint: disable=invalid-name
 )
-def test_puppet_target_09(mocker, tmp_path, pernosco, rr, valgrind):
+def test_puppet_target_09(
+    mocker, tmp_path, pernosco, rr, valgrind
+):  # pylint: disable=invalid-name
     """test PuppetTarget debugger args"""
     fake_ffp = mocker.patch("grizzly.target.puppet_target.FFPuppet", autospec=True)
     fake_file = tmp_path / "fake"
@@ -381,7 +383,7 @@ def test_puppet_target_10(tmp_path, asset, env):
             if env:
                 supp_env = tmp_path / "supp_env"
                 supp_env.touch()
-                target.environ["LSAN_OPTIONS"] = "suppressions='%s'" % (str(supp_env),)
+                target.environ["LSAN_OPTIONS"] = f"suppressions='{supp_env}'"
             else:
                 target.environ["LSAN_OPTIONS"] = "suppressions='missing'"
             target.process_assets()
@@ -409,3 +411,61 @@ def test_puppet_target_11(tmp_path):
         assert "TRACKED" in target.filtered_environ()
         assert "ASAN_OPTIONS" in target.filtered_environ()
         assert "LSAN_OPTIONS" not in target.filtered_environ()
+
+
+@mark.parametrize(
+    "base, extra, result",
+    [
+        # empty
+        ({}, {}, {}),
+        # empty extra
+        ({"a": "1"}, {}, {"a": "1"}),
+        # empty base
+        ({}, {"a": "1"}, {"a": "1"}),
+        # merge
+        ({"a": "1"}, {"b": "2"}, {"a": "1", "b": "2"}),
+        # name collision, favor base
+        ({"a": "1"}, {"a": "2"}, {"a": "1"}),
+        # name collision and merge
+        ({"a": "1"}, {"a": "2", "b": "2"}, {"a": "1", "b": "2"}),
+    ],
+)
+def test_puppet_target_12(tmp_path, base, extra, result):
+    """test PuppetTarget.merge_environment()"""
+    fake_file = tmp_path / "fake"
+    fake_file.touch()
+    with PuppetTarget(str(fake_file), 300, 25, 5000) as target:
+        target.environ = base
+        target.merge_environment(extra)
+        assert target.environ == result
+
+
+@mark.parametrize(
+    "base, extra, result",
+    [
+        # collision with existing option
+        ({"ASAN_OPTIONS": "a=1"}, {"ASAN_OPTIONS": "a=2"}, ["a=1"]),
+        # add option from extra
+        ({"ASAN_OPTIONS": "a=1"}, {"ASAN_OPTIONS": "b=2:c3"}, ["a=1", "b=2", "c=3"]),
+        # add option from extra
+        ({"ASAN_OPTIONS": "a=1:c=3"}, {"ASAN_OPTIONS": "b=2"}, ["a=1", "b=2", "c=3"]),
+    ],
+)
+def test_puppet_target_13(tmp_path, base, extra, result):
+    """test PuppetTarget.merge_environment() - merge sanitizer options"""
+    fake_file = tmp_path / "fake"
+    fake_file.touch()
+    with PuppetTarget(str(fake_file), 300, 25, 5000) as target:
+        target.environ = base
+        target.merge_environment(extra)
+        for opt in target.environ["ASAN_OPTIONS"].split(":"):
+            assert opt in result
+
+
+def test_puppet_target_14(mocker, tmp_path):
+    """test PuppetTarget.dump_coverage() - skip on unsupported platform"""
+    mocker.patch("grizzly.target.puppet_target.system", return_value="foo")
+    fake_file = tmp_path / "fake"
+    fake_file.touch()
+    with PuppetTarget(str(fake_file), 300, 25, 5000) as target:
+        target.dump_coverage()
