@@ -1,4 +1,3 @@
-# coding=utf-8
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
@@ -16,9 +15,9 @@ from .adapter import Adapter
 from .common.reporter import Report, Reporter
 from .common.runner import RunResult
 from .session import LogOutputLimiter, Session, SessionError
-from .target import AssetManager, Result, Target, TargetLaunchError
+from .target import AssetManager, Result, Target
 
-pytestmark = mark.usefixtures("patch_collector", "tmp_path_status_db")
+pytestmark = mark.usefixtures("patch_collector", "tmp_path_status_db_fuzz")
 
 
 class SimpleAdapter(Adapter):
@@ -71,7 +70,7 @@ def test_session_01(mocker, harness, profiling, coverage, relaunch, iters, runti
     target = mocker.Mock(
         spec_set=Target,
         assets=mocker.Mock(spec_set=AssetManager),
-        environ=dict(),
+        environ={},
         launch_timeout=30,
     )
     target.log_size.return_value = 1000
@@ -141,7 +140,7 @@ def test_session_02(mocker, harness, relaunch, remaining):
     target = mocker.Mock(
         spec_set=Target,
         assets=mocker.Mock(spec_set=AssetManager),
-        environ=dict(),
+        environ={},
         launch_timeout=30,
     )
     # calculate if the target is 'closed' based on relaunch
@@ -197,7 +196,7 @@ def test_session_03(mocker, tmp_path, harness, report_size, relaunch, iters, has
     target = mocker.Mock(
         spec_set=Target,
         assets=mocker.Mock(spec_set=AssetManager),
-        environ=dict(),
+        environ={},
         launch_timeout=30,
     )
     target.monitor.launches = 1
@@ -223,7 +222,7 @@ def test_session_03(mocker, tmp_path, harness, report_size, relaunch, iters, has
             log_fp.write("SEGV on unknown address 0x0 (pc 0x0 bp 0x0 sp 0x0 T0)\n")
             log_fp.write("    #0 0xbad000 in foo /file1.c:123:234\n")
             log_fp.write("    #1 0x1337dd in bar /file2.c:1806:19\n")
-    report = Report(str(log_path), "bin")
+    report = Report(log_path, "bin")
     target.create_report.return_value = report
     with Session(
         adapter, reporter, server, target, relaunch=relaunch, report_size=report_size
@@ -249,7 +248,7 @@ def test_session_04(mocker):
     target = mocker.Mock(
         spec_set=Target,
         assets=mocker.Mock(spec_set=AssetManager),
-        environ=dict(),
+        environ={},
         launch_timeout=30,
     )
     target.monitor.launches = 1
@@ -265,13 +264,14 @@ def test_session_05(mocker):
     target = mocker.Mock(
         spec_set=Target,
         assets=mocker.Mock(spec_set=AssetManager),
-        environ=dict(),
+        environ={},
         launch_timeout=30,
     )
     target.monitor.launches = 1
     with Session(SimpleAdapter(False), None, server, target) as session:
         with raises(SessionError, match="Please check Adapter and Target"):
             session.run([], 10)
+    assert target.handle_hang.call_count == 1
 
 
 @mark.parametrize(
@@ -296,7 +296,7 @@ def test_session_06(mocker, harness, report_size):
     target = mocker.Mock(
         spec_set=Target,
         assets=mocker.Mock(spec_set=AssetManager),
-        environ=dict(),
+        environ={},
         launch_timeout=30,
     )
     target.monitor.launches = 1
@@ -343,7 +343,7 @@ def test_session_07(mocker, srv_results, target_result, ignored, results):
         spec_set=Target,
         assets=mocker.Mock(spec_set=AssetManager),
         closed=True,
-        environ=dict(),
+        environ={},
         launch_timeout=30,
     )
     target.monitor.launches = 1
@@ -372,7 +372,7 @@ def test_session_08(mocker):
     adapter.IGNORE_UNSERVED = False
     server = mocker.Mock(spec_set=Sapphire, port=0x1337)
     target = mocker.Mock(
-        spec_set=Target, assets=mocker.Mock(spec_set=AssetManager), environ=dict()
+        spec_set=Target, assets=mocker.Mock(spec_set=AssetManager), environ={}
     )
     target.monitor.launches = 1
     with Session(adapter, None, server, target) as session:
@@ -387,27 +387,6 @@ def test_session_08(mocker):
 
 
 def test_session_09(mocker):
-    """test Session.run() handle TargetLaunchError"""
-    report = mocker.Mock(
-        spec_set=Report, major="major123", minor="minor456", crash_hash="123"
-    )
-    report.crash_info.createShortSignature.return_value = "[@ sig]"
-    runner = mocker.patch("grizzly.session.Runner", autospec=True)
-    runner.return_value.launch.side_effect = TargetLaunchError("test", report)
-    runner.return_value.startup_failure = True
-    adapter = mocker.Mock(spec_set=Adapter, remaining=None)
-    server = mocker.Mock(spec_set=Sapphire, port=0x1337)
-    target = mocker.Mock(spec_set=Target)
-    target.monitor.launches = 1
-    with Session(adapter, mocker.Mock(spec_set=Reporter), server, target) as session:
-        with raises(TargetLaunchError, match="test"):
-            session.run([], 10, iteration_limit=2)
-        assert session.status.iteration == 1
-        assert session.status.results.total == 1
-        assert session.status.ignored == 0
-
-
-def test_session_10(mocker):
     """test Session.run() report hang"""
     result = RunResult([], 60.0, status=Result.FOUND, timeout=True)
     result.attempted = True
@@ -421,7 +400,7 @@ def test_session_10(mocker):
     target = mocker.Mock(
         spec_set=Target,
         assets=mocker.Mock(spec_set=AssetManager),
-        environ=dict(),
+        environ={},
     )
     target.monitor.launches = 1
     target.create_report.return_value = report
@@ -446,7 +425,7 @@ def test_session_10(mocker):
         (True, 1, 1, 10, 5),
     ],
 )
-def test_session_11(mocker, harness, report_size, relaunch, iters, report_limit):
+def test_session_10(mocker, harness, report_size, relaunch, iters, report_limit):
     """test Session - limit report submission"""
     adapter = SimpleAdapter(harness)
     reporter = mocker.Mock(spec_set=Reporter)
@@ -456,7 +435,7 @@ def test_session_11(mocker, harness, report_size, relaunch, iters, report_limit)
     target = mocker.Mock(
         spec_set=Target,
         assets=mocker.Mock(spec_set=AssetManager),
-        environ=dict(),
+        environ={},
         launch_timeout=30,
     )
     target.monitor.launches = 1
@@ -503,7 +482,7 @@ def test_session_12(mocker, harness, iters, result_limit, results):
     target = mocker.Mock(
         spec_set=Target,
         assets=mocker.Mock(spec_set=AssetManager),
-        environ=dict(),
+        environ={},
         launch_timeout=30,
     )
     target.monitor.launches = 1
