@@ -1,4 +1,3 @@
-# coding=utf-8
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
@@ -42,8 +41,8 @@ class Sapphire:
     def __exit__(self, *exc):
         self.close()
 
-    @classmethod
-    def _create_listening_socket(cls, remote, port=None, retries=20):
+    @staticmethod
+    def _create_listening_socket(remote, port=None, retries=20, timeout=LISTEN_TIMEOUT):
         """Create listening socket. Search for an open socket if needed and
         and configure the socket. If a specific port is unavailable or no
         available ports can be found socket.error will be raised.
@@ -52,30 +51,36 @@ class Sapphire:
             remote (bool): Accept all (non-local) incoming connections.
             port (int): Port to listen on. If None is given a random port will
                         be used.
-            retries (int): Number of attempts to configure the socket.
+            retries (int): Number of additional attempts to configure the socket.
+            timeout (float): Used to set socket timeout.
 
         Returns:
             socket: A listening socket.
         """
+        assert retries >= 0
+        assert timeout > 0
         addr = "0.0.0.0" if remote else "127.0.0.1"
-        for retry in reversed(range(retries)):
-            sock = None
+        sock = None
+        for retry in reversed(range(max(retries + 1, 1))):
             try:
                 sock = socket(AF_INET, SOCK_STREAM)
                 sock.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)
-                sock.settimeout(cls.LISTEN_TIMEOUT)
+                sock.settimeout(timeout)
                 # find an unused port and avoid blocked ports
                 # see: searchfox.org/mozilla-central/source/netwerk/base/nsIOService.cpp
-                sock.bind((addr, port or randint(0x2000, 0xFFFF)))
+                # highest blocked port is 10080 (0x2760)
+                sock.bind((addr, port or randint(0x2770, 0xFFFF)))
                 sock.listen(5)
             except OSError as soc_e:
                 if sock is not None:
                     sock.close()
+                    sock = None
                 if retry > 1 and soc_e.errno in (EADDRINUSE, 10013):
                     sleep(0.1)
                     continue
                 raise
             break
+        assert sock is not None
         return sock
 
     def clear_backlog(self):
@@ -144,7 +149,7 @@ class Sapphire:
             - Served.REQUEST: Some files were requested
 
         Args:
-            path (str): Directory to use as wwwroot.
+            path (Path): Directory to use as wwwroot.
             continue_cb (callable): A callback that can be used to exit the serve loop.
                                     This must be a callable that returns a bool.
             forever (bool): Continue to handle requests even after all files have
@@ -156,8 +161,8 @@ class Sapphire:
         Returns:
             tuple(int, tuple(str)): Status code and files served.
         """
-        path = Path(path)
-        LOG.debug("serving %r (forever=%r)", str(path), forever)
+        assert isinstance(path, Path)
+        LOG.debug("serving '%s' (forever=%r)", path, forever)
         job = Job(
             path,
             auto_close=self._auto_close,
@@ -210,8 +215,8 @@ class Sapphire:
                 allow_remote=args.remote, port=args.port, timeout=args.timeout
             ) as serv:
                 LOG.info(
-                    "Serving %r @ http://%s:%d/",
-                    str(Path(args.path).resolve()),
+                    "Serving '%s' @ http://%s:%d/",
+                    args.path.resolve(),
                     gethostname() if args.remote else "127.0.0.1",
                     serv.port,
                 )
